#!/usr/bin/env perl
 
package NagiosListener;
 
use strict;
use warnings;
use Data::Dumper;
use Net::Server::PreFork;
our @ISA = qw(Net::Server::PreFork);
use Nagios::Cmd;
 
my $NAGSOCKET = '/var/lib/nagios3/rw/nagios.cmd';
my $PIDfile = "/var/run/naglisten.pid";
my $TIME = time();
 
if (not -w $NAGSOCKET) { die ("Unable to open $NAGSOCKET for writing\n"); }
 
my $nagios = Nagios::Cmd->new( $NAGSOCKET );
 
NagiosListener->run(port=>5777, user=>'nagios', group=>'www-data', background=>1);
 
open (PID, '>', $PIDfile) or
   die("Unable to open PID file for writing\n");
print PID $$ +1 ."\n";
close PID;
 
$SIG{'TERM'} = sub
{
    unlink $PIDfile or die "Unable to remove PID file: $PIDfile\n";
};
 
$SIG{'HUP'} = sub
{
    unlink $PIDfile or die "Unable to remove PID file: $PIDfile\n";
    exec($0, '') or die "Couldn't restart: $!\n";
};
 
sub process_request {
    my $self = shift;
    eval
    {
        local $SIG{'ALRM'} = sub { die "Timed Out!\n" };
        my $timeout = 300; # give the user 300 seconds to type some lines
 
        my $previous_alarm = alarm($timeout);
        while (<STDIN>)
        {
            s/\r?\n$//;
            my $col = trim($_);
 
            print STDERR "Command: $col\n";
 
            if ($col eq 'sysload')
            {
                my $load = trim(ptime |tr -s ' ' |cut -d ' ' -f 11 |cut -d , -f 1);
                print "$load\r\n";
                next;
            }
 
            # Check for a properly formatted response
            if ($col =~ /^[\w\.]+;[\w\s_]+;\d{1}?;.+/)
            {
                my ($hostname, $service, $status, $desc) = split /;/, $col;
 
                $nagios->service_check($hostname, $service, $status, $desc);
 
                print "OK\r\n";
                next;
            }
 
            print "ERROR: Invalid Command\r\n";
 
        }
 
        alarm($previous_alarm);
 
    };
 
    if ($@ =~ /timed out/i)
    {
           print STDOUT "Timed Out.\r\n";
           return;
    }
}
 
sub trim
{
    my $string = shift;
    if (not $string) { return $string }
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
}
 
